#!/usr/bin/python
# -*- coding: utf8 -*-
#  Convert Special Plugin
#
#  Copyright (C) 2012 Marco Crippa
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#from gettext import gettext as _
#import gedit
#import gtk
import string
from gi.repository import GObject, Gtk, Gdk, Gedit, Gio

ui_str = """
<ui>
  <menubar name="MenuBar">
	<menu name="EditMenu" action="Edit">
	  <placeholder name="EditOps_6">
	    <menu name="ConvertSpecial" action="ConvertSpecial">
	      <menuitem name="ToXML" action="ToXML"/>
	      <menuitem name="ToHTML" action="ToHTML"/>
	      <menuitem name="CancToXML" action="CancToXML"/>
	      <menuitem name="CancToHTML" action="CancToHTML"/>
	      <menuitem name="AmpToXML" action="AmpToXML"/>
	      <menuitem name="AmpToHTML" action="AmpToHTML"/>
	      <menuitem name="ChrToLatex" action="ChrToLatex"/>
	    </menu>
	  </placeholder>
	</menu>
  </menubar>
</ui>
"""

class ConvertSpecialPlugin(GObject.Object, Gedit.WindowActivatable):
	__gtype_name__ = "ConvertSpecial"
	window = GObject.property(type=Gedit.Window)

	set_chr = [ 

		['"', "&#34;", "&quot;"],
		["%", "&#37;", "&#37;"],
		["&", "&#38;", "&amp;"],
		["<", "&#60;", "&lt;"],
		[">", "&#62;", "&gt;"],
		["?", "&#63;", "&#63;"],
		["@", "&#64;", "&#64;"],
		["[", "&#91;", "&#91;"],
		["\\", "&#92;", "&#92;"],
		["]", "&#93;", "&#93;"],
		["^", "&#94;", "&#94;"],
		["_", "&#95;", "&#95;"],
		["`", "&#96;", "&#96;"],

		["{", "&#123;", "&#123;"],
		["|", "&#124;", "&#124;"],
		["}", "&#125;", "&#125;"],
		["~", "&#126;", "&#126;"],

		[" ", "&#160;", "&nbsp;"],
		["¡", "&#161;", "&iexcl;"],
		["¢", "&#162;", "&cent;"],
		["£", "&#163;", "&pound;"],
		["¤", "&#164;", "&curren;"],
		["¥", "&#165;", "&yen;"],
		["¦", "&#166;", "&brvbar;"],
		["§", "&#167;", "&sect;"],
		["¨", "&#168;", "&uml;"],
		["©", "&#169;", "&copy;"],
		["ª", "&#170;", "&ordf;"],
		["«", "&#171;", "&laquo;"],
		["¬", "&#172;", "&not;"],
		["®", "&#174;", "&reg;"],
		["¯", "&#175;", "&macr;"],
		["°", "&#176;", "&deg;"],
		["±", "&#177;", "&plusmn;"],
		["²", "&#178;", "&sup2;"],
		["³", "&#179;", "&sup3;"],
		["´", "&#180;", "&acute;"],
		["µ", "&#181;", "&micro;"],
		["¶", "&#182;", "&para;"],
		["·", "&#183;", "&middot;"],
		["¸", "&#184;", "&cedil;"],
		["¹", "&#185;", "&sup1;"],
		["º", "&#186;", "&ordm;"],
		["»", "&#187;", "&raquo;"],
		["¼", "&#188;", "&frac14;"],
		["½", "&#189;", "&frac12;"],
		["¾", "&#190;", "&frac34;"],
		["¿", "&#191;", "&iquest;"],
		["À", "&#192;", "&Agrave;"],
		["Á", "&#193;", "&Aacute;"],
		["Â", "&#194;", "&Acirc;"],
		["Ã", "&#195;", "&Atilde;"],
		["Ä", "&#196;", "&Auml;"],
		["Å", "&#197;", "&Aring;"],
		["Æ", "&#198;", "&AElig;"],
		["Ç", "&#199;", "&Ccedil;"],

		["È", "&#200;", "&Egrave;"],
		["É", "&#201;", "&Eacute;"],
		["Ê", "&#202;", "&Ecirc;"],
		["Ë", "&#203;", "&Euml;"],
		["Ì", "&#204;", "&Igrave;"],
		["Í", "&#205;", "&Iacute;"],
		["Î", "&#206;", "&Icirc;"],
		["Ï", "&#207;", "&Iuml;"],
		["Ð", "&#208;", "&ETH;"],
		["Ñ", "&#209;", "&Ntilde;"],
		["Ò", "&#210;", "&Ograve;"],
		["Ó", "&#211;", "&Oacute;"],
		["Ô", "&#212;", "&Ocirc;"],
		["Õ", "&#213;", "&Otilde;"],
		["Ö", "&#214;", "&Ouml;"],
		["×", "&#215;", "&times;"],
		["Ø", "&#216;", "&Oslash;"],
		["Ù", "&#217;", "&Ugrave;"],
		["Ú", "&#218;", "&Uacute;"],
		["Û", "&#219;", "&Ucirc;"],
		["Ü", "&#220;", "&Uuml;"],
		["Ý", "&#221;", "&Yacute;"],
		["Þ", "&#222;", "&THORN;"],
		["ß", "&#223;", "&szlig;"],
		["à", "&#224;", "&agrave;"],
		["á", "&#225;", "&aacute;"],
		["â", "&#226;", "&acirc;"],
		["ã", "&#227;", "&atilde;"],
		["ä", "&#228;", "&auml;"],
		["å", "&#229;", "&aring;"],
		["æ", "&#230;", "&aelig;"],
		["ç", "&#231;", "&ccedil;"],
		["è", "&#232;", "&egrave;"],
		["é", "&#233;", "&eacute;"],
		["ê", "&#234;", "&ecirc;"],
		["ë", "&#235;", "&euml;"],
		["ì", "&#236;", "&igrave;"],
		["í", "&#237;", "&iacute;"],
		["î", "&#238;", "&icirc;"],
		["ï", "&#239;", "&iuml;"],
		["ð", "&#240;", "&eth;"],
		["ñ", "&#241;", "&ntilde;"],
		["ò", "&#242;", "&ograve;"],
		["ó", "&#243;", "&oacute;"],
		["ô", "&#244;", "&ocirc;"],
		["õ", "&#245;", "&otilde;"],
		["ö", "&#246;", "&ouml;"],
		["÷", "&#247;", "&divide;"],
		["ø", "&#248;", "&oslash;"],
		["ù", "&#249;", "&ugrave;"],
		["ú", "&#250;", "&uacute;"],
		["û", "&#251;", "&ucirc;"],
		["ü", "&#252;", "&uuml;"],
		["ý", "&#253;", "&yacute;"],
		["þ", "&#254;", "&thorn;"],
		["ÿ", "&#255;", "&yuml;"],
		["Ā", "&#256;", "&#256;"],
		["ā", "&#257;", "&#257;"],
		["Ă", "&#258;", "&#258;"],
		["ă", "&#259;", "&#259;"],
		["Ą", "&#260;", "&#260;"],
		["ą", "&#261;", "&#261;"],
		["Ć", "&#262;", "&#262;"],
		["ć", "&#263;", "&#263;"],
		["Ĉ", "&#264;", "&#264;"],
		["ĉ", "&#265;", "&#265;"],
		["Ċ", "&#266;", "&#266;"],
		["ċ", "&#267;", "&#267;"],
		["Č", "&#268;", "&#268;"],
		["č", "&#269;", "&#269;"],
		["Ď", "&#270;", "&#270;"],
		["ď", "&#271;", "&#271;"],
		["Đ", "&#272;", "&#272;"],
		["đ", "&#273;", "&#273;"],
		["Ē", "&#274;", "&#274;"],
		["ē", "&#275;", "&#275;"],
		["Ĕ", "&#276;", "&#276;"],
		["ĕ", "&#277;", "&#277;"],
		["Ė", "&#278;", "&#278;"],
		["ė", "&#279;", "&#279;"],
		["Ę", "&#280;", "&#280;"],
		["ę", "&#281;", "&#281;"],
		["Ě", "&#282;", "&#282;"],
		["ě", "&#283;", "&#283;"],
		["Ĝ", "&#284;", "&#284;"],
		["ĝ", "&#285;", "&#285;"],
		["Ğ", "&#286;", "&#286;"],
		["ğ", "&#287;", "&#287;"],
		["Ġ", "&#288;", "&#288;"],
		["ġ", "&#289;", "&#289;"],
		["Ģ", "&#290;", "&#290;"],
		["ģ", "&#291;", "&#291;"],
		["Ĥ", "&#292;", "&#292;"],
		["ĥ", "&#293;", "&#293;"],
		["Ħ", "&#294;", "&#294;"],
		["ħ", "&#295;", "&#295;"],
		["Ĩ", "&#296;", "&#296;"],
		["ĩ", "&#297;", "&#297;"],
		["Ī", "&#298;", "&#298;"],
		["ī", "&#299;", "&#299;"],

		["İ", "&#304;", "&#304;"],
		["ı", "&#305;", "&#305;"],
		["Ĵ", "&#308;", "&#308;"],
		["ĵ", "&#309;", "&#309;"],
		["Ķ", "&#310;", "&#310;"],
		["ķ", "&#311;", "&#311;"],
		["Ĺ", "&#313;", "&#313;"],
		["ĺ", "&#314;", "&#314;"],
		["Ļ", "&#315;", "&#315;"],
		["ļ", "&#316;", "&#316;"],
		["Ľ", "&#317;", "&#317;"],
		["ľ", "&#318;", "&#318;"],
		["Ł", "&#321;", "&#321;"],
		["ł", "&#322;", "&#322;"],
		["Ń", "&#323;", "&#323;"],
		["ń", "&#324;", "&#324;"],
		["Ņ", "&#325;", "&#325;"],
		["ņ", "&#326;", "&#326;"],
		["Ň", "&#327;", "&#327;"],
		["ň", "&#328;", "&#328;"],
		["Ő", "&#336;", "&#336;"],
		["ő", "&#337;", "&#337;"],
		["Œ", "&#338;", "&OElig;"],
		["œ", "&#339;", "&oelig;"],
		["Ŕ", "&#340;", "&#340;"],
		["ŕ", "&#341;", "&#341;"],
		["Ŗ", "&#342;", "&#342;"],
		["ŗ", "&#343;", "&#343;"],
		["Ř", "&#344;", "&#344;"],
		["ř", "&#345;", "&#345;"],
		["Ś", "&#346;", "&#346;"],
		["ś", "&#347;", "&#347;"],
		["Ŝ", "&#348;", "&#348;"],
		["ŝ", "&#349;", "&#349;"],
		["Ş", "&#350;", "&#350;"],
		["ş", "&#351;", "&#351;"],
		["Š", "&#352;", "&Scaron;"],
		["š", "&#353;", "&scaron;"],
		["Ţ", "&#354;", "&#354;"],
		["ţ", "&#355;", "&#355;"],
		["Ť", "&#356;", "&#356;"],
		["ť", "&#357;", "&#357;"],
		["Ū", "&#362;", "&#362;"],
		["ū", "&#363;", "&#363;"],
		["Ŭ", "&#364;", "&#364;"],
		["ŭ", "&#365;", "&#365;"],
		["Ů", "&#366;", "&#366;"],
		["ů", "&#367;", "&#367;"],
		["Ű", "&#368;", "&#368;"],
		["ű", "&#369;", "&#369;"],
		["Ÿ", "&#376;", "&Yuml;"],
		["Ź", "&#377;", "&#377;"],
		["ź", "&#378;", "&#378;"],
		["Ż", "&#379;", "&#379;"],
		["ż", "&#380;", "&#380;"],
		["Ž", "&#381;", "&#381;"],
		["ž", "&#382;", "&#382;"],

		["ƒ", "&#402;", "&fnof;"],

		["ˆ", "&#710;", "&circ;"],
		["˜", "&#732;", "&tilde;"],

		["Α", "&#913;", "&Alpha;"],
		["Β", "&#914;", "&Beta;"],
		["Γ", "&#915;", "&Gamma;"],
		["Δ", "&#916;", "&Delta;"],
		["Ε", "&#917;", "&Epsilon;"],
		["Ζ", "&#918;", "&Zeta;"],
		["Η", "&#919;", "&Eta;"],
		["Θ", "&#920;", "&Theta;"],
		["Ι", "&#921;", "&Iota;"],
		["Κ", "&#922;", "&Kappa;"],
		["Λ", "&#923;", "&Lambda;"],
		["Μ", "&#924;", "&Mu;"],
		["Ν", "&#925;", "&Nu;"],
		["Ξ", "&#926;", "&Xi;"],
		["Ο", "&#927;", "&Omicron;"],
		["Π", "&#928;", "&Pi;"],
		["Ρ", "&#929;", "&Rho;"],
		["Σ", "&#931;", "&Sigma;"],
		["Τ", "&#932;", "&Tau;"],
		["Υ", "&#933;", "&Upsilon;"],
		["Φ", "&#934;", "&Phi;"],
		["Χ", "&#935;", "&Chi;"],
		["Ψ", "&#936;", "&Psi;"],
		["Ω", "&#937;", "&Omega;"],
		["α", "&#945;", "&alpha;"],
		["β", "&#946;", "&beta;"],
		["γ", "&#947;", "&gamma;"],
		["δ", "&#948;", "&delta;"],
		["ε", "&#949;", "&epsilon;"],
		["ζ", "&#950;", "&zeta;"],
		["η", "&#951;", "&eta;"],
		["θ", "&#952;", "&theta;"],
		["ι", "&#953;", "&iota;"],
		["κ", "&#954;", "&kappa;"],
		["λ", "&#955;", "&lambda;"],
		["μ", "&#956;", "&mu;"],
		["ν", "&#957;", "&nu;"],
		["ξ", "&#958;", "&xi;"],
		["ο", "&#959;", "&omicron;"],
		["π", "&#960;", "&pi;"],
		["ρ", "&#961;", "&rho;"],
		["ς", "&#962;", "&sigmaf;"],
		["σ", "&#963;", "&sigma;"],
		["τ", "&#964;", "&tau;"],
		["υ", "&#965;", "&upsilon;"],
		["φ", "&#966;", "&phi;"],
		["χ", "&#967;", "&chi;"],
		["ψ", "&#968;", "&psi;"],
		["ω", "&#969;", "&omega;"],
		["ϑ", "&#977;", "&thetasym;"],
		["ϒ", "&#978;", "&upsih;"],
		["ϖ", "&#982;", "&piv;"],

		["–", "&#8211;", "&ndash;"],
		["—", "&#8212;", "&mdash;"],
		["‘", "&#8216;", "&lsquo;"],
		["’", "&#8217;", "&rsquo;"],
		["‚", "&#8218;", "&sbquo;"],
		["“", "&#8220;", "&ldquo;"],
		["”", "&#8221;", "&rdquo;"],
		["„", "&#8222;", "&bdquo;"],
		["†", "&#8224;", "&dagger;"],
		["‡", "&#8225;", "&Dagger;"],
		["•", "&#8226;", "&bull;"],
		["…", "&#8230;", "&hellip;"],
		["‰", "&#8240;", "&permil;"],
		["′", "&#8242;", "&prime;"],
		["″", "&#8243;", "&Prime;"],
		["‹", "&#8249;", "&lsaquo;"],
		["›", "&#8250;", "&rsaquo;"],
		["‾", "&#8254;", "&oline;"],

		["€", "&#8364;", "&euro;"],

		["№", "&#8470;", "&#8470;"],
		["™", "&#8482;", "&trade;"],

		["←", "&#8592;", "&larr;"],
		["↑", "&#8593;", "&uarr;"],
		["→", "&#8594;", "&rarr;"],
		["↓", "&#8595;", "&darr;"],
		["↔", "&#8596;", "&harr;"],

		["↵", "&#8629;", "&crarr;"],

		["∀", "&#8704;", "&forall;"],
		["∂", "&#8706;", "&part;"],
		["∃", "&#8707;", "&exists;"],
		["∅", "&#8709;", "&empty;"],
		["∇", "&#8711;", "&nabla;"],
		["∈", "&#8712;", "&isin;"],
		["∉", "&#8713;", "&notin;"],
		["∋", "&#8715;", "&ni;"],
		["∏", "&#8719;", "&prod;"],
		["∑", "&#8721;", "&sum;"],
		["−", "&#8722;", "&minus;"],
		["∗", "&#8727;", "&lowast;"],
		["√", "&#8730;", "&radic;"],
		["∝", "&#8733;", "&prop;"],
		["∞", "&#8734;", "&infin;"],
		["∠", "&#8736;", "&ang;"],
		["∧", "&#8743;", "&and;"],
		["∨", "&#8744;", "&or;"],
		["∩", "&#8745;", "&cap;"],
		["∪", "&#8746;", "&cup;"],
		["∫", "&#8747;", "&int;"],
		["∴", "&#8756;", "&there4;"],
		["∼", "&#8764;", "&sim;"],
		["≅", "&#8773;", "&cong;"],
		["≈", "&#8776;", "&asymp;"],

		["≠", "&#8800;", "&ne;"],
		["≡", "&#8801;", "&equiv;"],
		["≤", "&#8804;", "&le;"],
		["≥", "&#8805;", "&ge;"],
		["⊂", "&#8834;", "&sub;"],
		["⊃", "&#8835;", "&sup;"],
		["⊄", "&#8836;", "&nsub;"],
		["⊆", "&#8838;", "&sube;"],
		["⊇", "&#8839;", "&supe;"],
		["⊕", "&#8853;", "&oplus;"],
		["⊗", "&#8855;", "&otimes;"],
		["⊥", "&#8869;", "&perp;"],

		["⋅", "&#8901;", "&sdot;"],
		["⌈", "&#8968;", "&lceil;"],
		["⌉", "&#8969;", "&rceil;"],
		["⌊", "&#8970;", "&lfloor;"],
		["⌋", "&#8971;", "&rfloor;"],

		["◊", "&#9674;", "&loz;"],

		["♠", "&#9824;", "&spades;"],
		["♣", "&#9827;", "&clubs;"],
		["♥", "&#9829;", "&hearts;"],
		["♦", "&#9830;", "&diams;"]
	]


	set_chr_latex = [ 
		["à","\`a"],
		["è","\`e"],
		["é","\\'e"],
		["ì","\`i"],
		["ò","\`o"],
		["ù","\`u"],
		["°","$^{\circ}$"],
		["%","\%"]
	]

	def __init__(self):
		GObject.Object.__init__(self)

	def do_activate(self):
		self._window = self.window
		action = Gio.SimpleAction(name="gedit-convert-XML-plugin")
		action.connect("activate", self.ToXML)
		self._window.add_action(action)
		action = Gio.SimpleAction(name="gedit-convert-HTML-plugin")
		action.connect("activate", self.ToHTML)
		self._window.add_action(action)
		action = Gio.SimpleAction(name="gedit-convert-#-XML-plugin")
		action.connect("activate", self.CancToHTML_XML)
		self._window.add_action(action)
		action = Gio.SimpleAction(name="gedit-convert-#-HTML-plugin")
		action.connect("activate", self.CancToHTML_XML)
		self._window.add_action(action)
		action = Gio.SimpleAction(name="gedit-convert-&-XML-plugin")
		action.connect("activate", self.AmpToXML)
		self._window.add_action(action)
		action = Gio.SimpleAction(name="gedit-convert-&-HTML-plugin")
		action.connect("activate", self.AmpToHTML)
		self._window.add_action(action)

	def do_deactivate(self):
		self._window.remove_action("gedit-convert-XML-plugin")
		self._window.remove_action("gedit-convert-HTML-plugin")

	def do_update_state(self):
		view = self._window.get_active_view()

	def ToXML(self, action, xxx):
		doc = self._window.get_active_document()
		bounds = doc.get_selection_bounds()
		if not bounds: return
		text = doc.get_text(bounds[0], bounds[1], False)
		for a in range (0,len(self.set_chr)):
			text=text.replace(self.set_chr[a][0],self.set_chr[a][1])
		doc.begin_user_action()
		doc.delete(*bounds)
		doc.insert_at_cursor(text)
		doc.end_user_action()

	def ToHTML(self, action, xxx):
		doc = self._window.get_active_document()
		bounds = doc.get_selection_bounds()
		if not bounds: return
		text = doc.get_text(bounds[0], bounds[1], False)
		for a in range (0,len(self.set_chr)):
			text=text.replace(self.set_chr[a][0],self.set_chr[a][2])
		doc.begin_user_action()
		doc.delete(*bounds)
		doc.insert_at_cursor(text)
		doc.end_user_action()

	def CancToHTML_XML(self, action, xxx):
		doc = self._window.get_active_document()
		bounds = doc.get_selection_bounds()
		if not bounds: return
		text = doc.get_text(bounds[0], bounds[1], False)
		text=text.replace("#","&#35;")
		doc.begin_user_action()
		doc.delete(*bounds)
		doc.insert_at_cursor(text)
		doc.end_user_action()

	def AmpToXML(self, action, xxx):
		doc = self._window.get_active_document()
		bounds = doc.get_selection_bounds()
		if not bounds: return
		text = doc.get_text(bounds[0], bounds[1], False)
		text=text.replace("&","&#38;")
		doc.begin_user_action()
		doc.delete(*bounds)
		doc.insert_at_cursor(text)
		doc.end_user_action()

	def AmpToHTML(self, action, xxx):
		doc = self._window.get_active_document()
		bounds = doc.get_selection_bounds()
		if not bounds: return
		text = doc.get_text(bounds[0], bounds[1], False)
		text=text.replace("&","&amp;")
		doc.begin_user_action()
		doc.delete(*bounds)
		doc.insert_at_cursor(text)
		doc.end_user_action()

	def ChrToLatex(self, action):
		doc = self._window.get_active_document()
		bounds = doc.get_selection_bounds()
		if not bounds: return
		text = doc.get_text(bounds[0], bounds[1], False)
		for a in range (0,len(self.set_chr_latex)):
			text=text.replace(self.set_chr_latex[a][0],self.set_chr_latex[a][1])
		doc.begin_user_action()
		doc.delete(*bounds)
		doc.insert_at_cursor(text)
		doc.end_user_action()

class ConvertSpecialAppHelper(GObject.Object, Gedit.AppActivatable):
    __gtype_name__ = "ConvertSpecialAppHelper"
    app = GObject.property(type=Gedit.App)

    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        if hasattr(self, "extend_menu"):
            self.app.add_accelerator("<control><alt>x", "win.gedit-convert-XML-plugin", None)
            self.app.add_accelerator("<control><alt>h", "win.gedit-convert-HTML-plugin", None)

            self.menu_ext = self.extend_menu("search-section")
            item = Gio.MenuItem.new(_("Convert to XML"), "win.gedit-convert-XML-plugin")
            self.menu_ext.append_menu_item(item)
            item = Gio.MenuItem.new(_("Convert to HTML"), "win.gedit-convert-HTML-plugin")
            self.menu_ext.append_menu_item(item)
            item = Gio.MenuItem.new(_("Convert # to XML"), "win.gedit-convert-#-XML-plugin")
            self.menu_ext.append_menu_item(item)
            item = Gio.MenuItem.new(_("Convert # to HTML"), "win.gedit-convert-#-HTML-plugin")
            self.menu_ext.append_menu_item(item)
            item = Gio.MenuItem.new(_("Convert & to XML"), "win.gedit-convert-&-XML-plugin")
            self.menu_ext.append_menu_item(item)
            item = Gio.MenuItem.new(_("Convert & to HTML"), "win.gedit-convert-&-HTML-plugin")
            self.menu_ext.append_menu_item(item)

    def do_deactivate(self):
        if hasattr(self, "extend_menu"):
            self.app.remove_accelerator("win.gedit-convert-XML-plugin", None)
            self.app.remove_accelerator("win.gedit-convert-HTML-plugin", None)
            self.menu_ext = None



